@@ -20,11 +20,10 @@
 ##
 ##############################################################################
 
-C_version="6.60-1"
+C_version="6.70-1"
 C_copyright_end="2023"
 C_default_system="deb"
-
-P_support_printer=("G3070" "G2070" "G4070" "G1030" "G3030" "G2030")
+P_support_printer=("TS8700" "XK120" "GX5500" "GX1000" "GX2000" "TS6630" "TS7700" "TS7600i" "TS6730" "TR7800" "TS7700i" "TS7700A" "GX6500")
 
 L_INST_COM_01_01="Command executed = %s\n"
 
