@@ -52,6 +52,10 @@ static const MapTbl mediatypeTbl[] = {
 	{ "photo", CNCL_PSET_MEDIA_OTHER_PHOTO_PAPER },
 	{ "greetingcard", CNCL_PSET_MEDIA_GREETING_CARD },
 	{ "cardstock", CNCL_PSET_MEDIA_CARDSTOCK },
+	{ "medicine", CNCL_PSET_MEDIA_MEDICINE_ENVELOPE },
+	{ "thin", CNCL_PSET_MEDIA_THIN_PAPER },
+	{ "deliveryslip", CNCL_PSET_MEDIA_DELIVERY_SLIP },
+	{ "glossylabel", CNCL_PSET_MEDIA_GLOSSY_LABEL },
 	{ NULL, -1 },
 };
 
@@ -68,6 +72,8 @@ static const MapTbl papersizeTbl[] = {
 	{ "A3.bl", CNCL_PSET_SIZE_A3 },
 	{ "a3plus", CNCL_PSET_SIZE_A3_PLUS },
 	{ "a3plus.bl", CNCL_PSET_SIZE_A3_PLUS },
+	{ "B6", CNCL_PSET_SIZE_B6 },
+	{ "B6H", CNCL_PSET_SIZE_B6H },
 	{ "B5", CNCL_PSET_SIZE_B5 },
 	{ "B5.bl", CNCL_PSET_SIZE_B5 },
 	{ "B4", CNCL_PSET_SIZE_B4 },
