@@ -51,6 +51,10 @@
 #define CNCL_PSET_MEDIA_OTHER_PHOTO_PAPER (16)
 #define CNCL_PSET_MEDIA_GREETING_CARD (25)
 #define CNCL_PSET_MEDIA_CARDSTOCK (26)
+#define CNCL_PSET_MEDIA_MEDICINE_ENVELOPE (30)
+#define CNCL_PSET_MEDIA_THIN_PAPER (31)
+#define CNCL_PSET_MEDIA_DELIVERY_SLIP (32)
+#define CNCL_PSET_MEDIA_GLOSSY_LABEL (33)
 
 /* pagesize */
 #define CNCL_PSET_SIZE_LETTER (1)
@@ -61,6 +65,8 @@
 #define CNCL_PSET_SIZE_A4 (4)
 #define CNCL_PSET_SIZE_A3 (5)
 #define CNCL_PSET_SIZE_A3_PLUS (6)
+#define CNCL_PSET_SIZE_B6 (83)
+#define CNCL_PSET_SIZE_B6H (84)
 #define CNCL_PSET_SIZE_B5 (7)
 #define CNCL_PSET_SIZE_B4 (8)
 #define CNCL_PSET_SIZE_OFICIO (24)
