@@ -46,11 +46,22 @@
 #define IS_NUMBER(c)	(c >= '0' && c <= '9')
 // #define UUID_LEN	(37)
 
+#define CNIJ_TEMP "/var/tmp/cnijcachetmpXXXXXX"
+#define OPTION_TRUE "true"
+
+#define COLOR_MODE_COUNT 2
+
 // #include "ivec.h"
 #include "cnijutil.h"
 
 int (*CNCL_GetString)(const char*, const char*, int, uint8_t**);
-
+int WriteHeader(int fd, char jobID[], char uuid[], CNCL_P_SETTINGSPTR Settings, CAPABILITY_DATA capability);
+int WriteData(int in_fd, int out_fds[], char jobID[], enum ColorMode *jobColorMode);
+int WritePages(int in_fd, int out_fd, char jobID[]);
+int WriteTail(int out_fd, char jobID[]);
+void CreateCacheFile(int out_fds[]);
+int ReplayCacheFile(int in_fd, int out_fd);
+int GetJobId(char jobID[], CAPABILITY_DATA capability);
 
 enum {
 	OPT_VERSION = 0,
@@ -61,7 +72,9 @@ enum {
 	OPT_COLORMODE,
 	OPT_DUPLEXPRINT,
 	OPT_JOBID,
-	OPT_UUID
+	OPT_UUID,
+	OPT_ROTATE180,
+	OPT_OPTIMIZATION
 };
 
 static int is_size_X(char *str)
@@ -273,16 +286,355 @@ onErr:
 	return result;
 }
 
+int WriteHeader(int fd, char jobID[], char uuid[], CNCL_P_SETTINGSPTR Settings, CAPABILITY_DATA capability)
+{
+	DEBUG_PRINT( "[tocanonij] WriteHeader\n");
+	uint8_t *xmlBuf = NULL;
+	int xmlBufSize;
+	int writtenSize = 0;
+	long writtenSize_long = 0;
+	const char *p_ppd_name = getenv("PPD");
+
+	long bufSize = sizeof(char) * CN_BUFSIZE;
+	char *bufTop = NULL;
+	char *tmpBuf = NULL;
+
+	if ( (bufTop = malloc( bufSize )) == NULL ){
+		return -1;
+	}
+
+	int prot = GetProtocol( (char *)capability.deviceID, capability.deviceIDLength );
+
+	if( prot == 2 ){
+		xmlBufSize = GETSTRINGWITHTAGFROMFILE( p_ppd_name, CNCL_FILE_TAG_CAPABILITY, (int *)CNCL_DECODE_EXEC, &xmlBuf );
+
+		unsigned short hostEnv = 0;
+		hostEnv = ParseCapabilityResponsePrint_HostEnv( xmlBuf, xmlBufSize );
+
+		/* Write StartJob Command */
+		int ret = 0;
+		ret = MakeCommand_StartJob3( hostEnv, uuid, jobID, bufTop, bufSize, &writtenSize );
+
+		if ( ret != 0 ) {
+			fprintf( stderr, "Error in CNCL_GetPrintCommand\n" );
+			free(bufTop);
+			return -1;
+		}
+
+		/* WriteData */
+		if (  write( fd, bufTop, writtenSize ) != writtenSize ){
+			free(bufTop);
+			return -1;
+		} 
+
+		char dateTime[15];
+		memset(dateTime, '\0', sizeof(dateTime));
+
+		ret = ParseCapabilityResponsePrint_DateTime( xmlBuf, xmlBufSize );
+
+		if( ret == 2 ){
+			time_t timer = time(NULL);
+			struct tm *date = localtime(&timer);
+
+			sprintf(dateTime, "%d%02d%02d%02d%02d%02d",
+				date->tm_year+1900, date->tm_mon+1, date->tm_mday,
+				date->tm_hour, date->tm_min, date->tm_sec);
+
+			if ( (tmpBuf = malloc( bufSize )) == NULL ){
+				free(tmpBuf);
+				return -1;
+			}
+
+			MakeCommand_SetJobConfiguration( jobID, dateTime, tmpBuf, bufSize, &writtenSize );
+
+			/* WriteData */
+			if ( write( fd, tmpBuf, writtenSize ) != writtenSize ){
+				free(bufTop);
+				free(tmpBuf);
+				return -1;
+			}
+			
+			free(tmpBuf);
+			// writtenSize += tmpWrittenSize;
+		}
+	}
+	else{
+		/* OutputSetTime */
+		if ( OutputSetTime( fd, jobID ) != 0 ){
+			free(bufTop);
+			return -1;
+		}
+
+		/* Write StartJob Command */
+		if ( GETPRINTCOMMAND( bufTop, bufSize, &writtenSize_long, jobID, CNCL_COMMAND_START1 ) != 0 ) {
+			fprintf( stderr, "Error in CNCL_GetPrintCommand\n" );
+			free(bufTop);
+			return -1;
+		}
+
+		/* WriteData */
+		if ( write( fd, bufTop, writtenSize_long ) != writtenSize_long ){
+			free(bufTop);
+			return -1;
+		}
+	}
+
+	/* Write SetConfiguration Command */
+	if ( (xmlBufSize = GETSTRINGWITHTAGFROMFILE( p_ppd_name, CNCL_FILE_TAG_CAPABILITY, (int*)CNCL_DECODE_EXEC, &xmlBuf )) < 0 ){
+		DEBUG_PRINT2( "[tocanonij] p_ppd_name : %s\n", p_ppd_name );
+		DEBUG_PRINT2( "[tocanonij] xmlBufSize : %d\n", xmlBufSize );
+		fprintf( stderr, "Error in CNCL_GetStringWithTagFromFile\n" );
+		free(bufTop);
+		return -1;
+	}
+
+	if ( GETSETCONFIGURATIONCOMMAND( Settings, jobID, bufSize, (void *)xmlBuf, xmlBufSize, bufTop, &writtenSize_long ) != 0 ){
+		fprintf( stderr, "Error in CNCL_GetSetConfigurationCommand\n" );
+		free(bufTop);
+		return -1;
+	}
+	/* WriteData */
+	write( fd, bufTop, writtenSize_long );
+	free(bufTop);
+	return 0;
+}	
+
+int WriteData(int in_fd, int out_fds[], char jobID[], enum ColorMode *jobColorMode)
+{
+	long bufSize = sizeof(char) * CN_BUFSIZE;
+	char *bufTop = NULL;
+	long writtenSize_long = 0;
+
+	if ( (bufTop = malloc( bufSize )) == NULL ){
+		return -1;
+	}
+
+	while ( 1 ) {
+		int readBytes = 0;
+		int writeBytes;
+		CNDATA CNData;
+		long readSize = 0;
+		unsigned short next_page;
+		int out_fd = -1;
+
+		memset( &CNData, 0, sizeof(CNDATA) );
+
+		/* read magic number */
+		readBytes = read( in_fd, &CNData, sizeof(CNDATA) );
+		if ( readBytes > 0 ){
+			if ( CNData.magic_num != MAGIC_NUMBER_FOR_CNIJPWG ){
+				fprintf( stderr, "Error illeagal MagicNumber\n" );
+				free(bufTop);
+				return -1;
+			}
+			if ( CNData.image_size < 0 ){
+				fprintf( stderr, "Error illeagal dataSize\n" );
+				free(bufTop);
+				return -1;
+			}
+		}
+		else if ( readBytes < 0 ){
+			if ( errno == EINTR ) continue;
+			fprintf( stderr, "DEBUG:[tocanonij] tocnij read error, %d\n", errno );
+			free(bufTop);
+			return -1;
+		}
+		else {
+			DEBUG_PRINT( "DEBUG:[tocanonij] !!!DATA END!!!\n" );
+			break; /* data end */
+		}
+
+		if(jobColorMode != NULL) {
+			*jobColorMode = CNData.jobColorMode;
+		}	
+
+
+		if(CNData.pageColorMode == COLOR_MODE_COLOR) {
+			out_fd = out_fds[0];
+		} else if(CNData.pageColorMode == COLOR_MODE_GRAY) {
+			out_fd = out_fds[1];
+		}
+
+		/* Write Next Page Info */
+		if ( CNData.next_page ) {
+			next_page = CNCL_PSET_NEXTPAGE_ON;
+		}
+		else {
+			next_page = CNCL_PSET_NEXTPAGE_OFF; 
+		}
+		if ( GETSETPAGECONFIGUARTIONCOMMAND( jobID, next_page, bufTop, bufSize, &writtenSize_long ) != 0 ) {
+			fprintf( stderr, "Error in CNCL_GetPrintCommand\n" );
+			free(bufTop);
+			return -1;
+		}
+		
+		/* WriteData */
+		if ( write( out_fd, bufTop, writtenSize_long) != writtenSize_long ){
+			free(bufTop);
+			return -1;
+		} 
+
+		DEBUG_PRINT( "[tocanonij] Write SendData Command\n");
+		/* Write SendData Command */
+		memset(	bufTop, 0x00, bufSize );
+		readSize = CNData.image_size;
+		if ( GETSENDDATAPWGRASTERCOMMAND( jobID, readSize, bufSize, bufTop, &writtenSize_long ) != 0 ) {
+			DEBUG_PRINT( "Error in CNCL_GetSendDataJPEGCommand\n" );
+			free(bufTop);
+			return -1;
+		}
+		/* WriteData */
+		write( out_fd, bufTop, writtenSize_long );
+
+		while( readSize ){
+			char *pCurrent = bufTop;
+
+			if ( readSize - bufSize > 0 ){
+				readBytes = read( in_fd, bufTop, bufSize );
+				DEBUG_PRINT2( "[tocanonij] PASS tocanonij READ1<%d>\n", readBytes );
+				if ( readBytes < 0 ) {
+					if ( errno == EINTR ) continue;
+				}
+				readSize -= readBytes;
+			}
+			else {
+				readBytes = read( in_fd, bufTop, readSize );
+				DEBUG_PRINT2( "[tocanonij] PASS tocanonij READ2<%d>\n", readBytes );
+				if ( readBytes < 0 ) {
+					if ( errno == EINTR ) continue;
+				}
+				readSize -= readBytes;
+			}
+
+			do {
+				writeBytes = write( out_fd, pCurrent, readBytes );
+				DEBUG_PRINT2( "[tocanonij] PASS tocanonij WRITE<%d>\n", writeBytes );
+				if( writeBytes < 0){
+					if ( errno == EINTR ) continue;
+					free(bufTop);
+					return -1;
+				}
+				readBytes -= writeBytes;
+				pCurrent += writeBytes;
+			} while( writeBytes > 0 );
+		}
+	}
+
+	free(bufTop);
+
+	return 0;
+}
+
+int WritePages(int in_fd, int out_fd, char jobID[])
+{
+	DEBUG_PRINT( "[tocanonij] WritePages\n" );
+	int out_fds[2] = { out_fd, out_fd };
+	return WriteData(in_fd, out_fds, jobID, NULL);
+}
+
+int WriteCacheFile(int in_fd, int out_fds[], char jobID[], enum ColorMode *jobColorMode)
+{
+	DEBUG_PRINT( "[tocanonij] WriteCacheFile\n" );
+	return WriteData(in_fd, out_fds, jobID, jobColorMode);
+}
+
+int WriteTail(int out_fd, char jobID[])
+{
+	DEBUG_PRINT( "[tocanonij] WriteTail\n");
+	long bufSize = sizeof(char) * CN_BUFSIZE;
+	char *bufTop = NULL;
+	int retSize = 0;
+	long writtenSize_long = 0;
+
+	if ( (bufTop = malloc( bufSize )) == NULL ){
+		DEBUG_PRINT( "Error in malloc\n" );
+		return -1;
+	}
+
+	/* CNCL_GetPrintCommand */
+	if ( GETPRINTCOMMAND( bufTop, bufSize, &writtenSize_long, jobID, CNCL_COMMAND_END ) != 0 ) {
+		DEBUG_PRINT( "Error in CNCL_GetPrintCommand\n" );
+		free(bufTop);
+		return -1;
+	}
+	/* WriteData */
+	retSize = write( out_fd, bufTop, writtenSize_long );
+	if(retSize == 0){
+		DEBUG_PRINT( "Error in WriteData\n" );
+		free(bufTop);
+		return -1;
+	}
+	DEBUG_PRINT( "[tocanonij] to_cnijf <end>\n" );
+
+	free(bufTop);
+	return 0;
+}
+
+void CreateCacheFile(int out_fds[])
+{
+	DEBUG_PRINT( "[tocanonij] CreateCacheFile\n" );
+	for(int i = 0; i < COLOR_MODE_COUNT; i++){
+		char tmpName[64];
+		strncpy( tmpName, CNIJ_TEMP, 64 );
+		out_fds[i] = mkstemp( tmpName );
+		if(out_fds[i] != -1){
+			unlink( tmpName );
+		}
+	}
+}
+
+int ReplayCacheFile(int in_fd, int out_fd)
+{
+	DEBUG_PRINT( "[tocanonij] ReplayCacheFile\n");
+	long bufSize = sizeof(char) * CN_BUFSIZE;
+	char *bufTop = NULL;
+
+	if ( (bufTop = malloc( bufSize )) == NULL ){
+		DEBUG_PRINT( "Error in malloc\n" );
+		return -1;
+	}
+
+	lseek(in_fd, 0, SEEK_SET);
+	while(1){
+		int readBytes = read( in_fd, bufTop, bufSize );
+		if(readBytes <= 0){
+			break;
+		}
+
+		if(write(out_fd, bufTop, readBytes) == 0){
+			DEBUG_PRINT( "Error in WriteData\n" );
+			free( bufTop );
+			return -1;
+		}	
+	}
+	
+	free( bufTop );
+
+	DEBUG_PRINT( "ReplayCacheFile return 0\n" );
+	return 0;
+}
+
+int GetJobId(char jobID[], CAPABILITY_DATA capability)
+{
+	//GetJobId
+	int prot = GetProtocol( (char *)capability.deviceID, capability.deviceIDLength );
+	if(prot == 2){
+		/* Set JobID */
+		strncpy( jobID, CN_START_JOBID2, CN_START_JOBID_LEN );
+	}
+	else{
+		/* Set JobID */
+		strncpy( jobID, CN_START_JOBID, CN_START_JOBID_LEN );
+	}
+
+	return 0;
+}
+
 int main( int argc, char *argv[] )
 {
 	int fd = 0;
 	int opt, opt_index;
 	int result = -1;
-	long bufSize = 0; 
-	// long writtenSize;
-	int writtenSize = 0;
-	long writtenSize_long = 0;
-	char *bufTop = NULL;
 	CNCL_P_SETTINGS Settings;
 	char jobID[CN_START_JOBID_LEN];
 	char libPathBuf[CN_LIB_PATH_LEN];
@@ -296,17 +648,19 @@ int main( int argc, char *argv[] )
 		{ "duplexprint", required_argument, NULL, OPT_DUPLEXPRINT }, 
 		{ "jobid", required_argument, NULL, OPT_JOBID }, 
 		{ "uuid", required_argument, NULL, OPT_UUID }, 
+		{ "rotate180", required_argument, NULL, OPT_ROTATE180 },
+		{ "optimization", required_argument, NULL, OPT_OPTIMIZATION},
 		{ 0, 0, 0, 0 }, 
 	};
 	const char *p_ppd_name = getenv("PPD");
-	uint8_t *xmlBuf = NULL;
-	int xmlBufSize;
-	int retSize;
-
-	char *tmpBuf = NULL;
-
+	CAPABILITY_DATA capability;
 	char	uuid[UUID_LEN + 1];
 
+	short optimization = 0;
+	enum ColorMode jobColorMode = COLOR_MODE_GRAY;
+	int cnijtmp_fd = -1;
+	int cnijtmp_fds[2] = { -1, -1 };
+
 	DEBUG_PRINT( "[tocanonij] start tocanonij\n" );
 
 	/* init CNCL API */
@@ -380,7 +734,14 @@ int main( int argc, char *argv[] )
 					strncpy( uuid, optarg, strlen(optarg) );
 				}
 				break;
-
+			case OPT_ROTATE180:  /* ignore this option */
+				break;
+			case OPT_OPTIMIZATION:
+				DEBUG_PRINT3( "[tocanonij] OPTION(%s):VALUE(%s)\n", long_opt[opt_index].name, optarg );
+				if(strcasecmp(optarg, OPTION_TRUE) == 0){
+					optimization = 1;
+				}
+				break;
 			case '?':
 				fprintf( stderr, "Error: invalid option %c:\n", optopt);
 				break;
@@ -402,291 +763,153 @@ int main( int argc, char *argv[] )
 	libclss = dlopen( libPathBuf, RTLD_LAZY );
 	if ( !libclss ) {
 		fprintf( stderr, "Error in dlopen\n" );
-		goto onErr1;
+		goto onErr;
 	}
 
 	GETSETCONFIGURATIONCOMMAND = dlsym( libclss, "CNCL_GetSetConfigurationCommand" );
 	if ( dlerror() != NULL ) {
 		fprintf( stderr, "Error in CNCL_GetSetConfigurationCommand. API not Found.\n" );
-		goto onErr2;
+		goto onErr;
 	}
 	GETSENDDATAPWGRASTERCOMMAND = dlsym( libclss, "CNCL_GetSendDataPWGRasterCommand" );
 	if ( dlerror() != NULL ) {
 		fprintf( stderr, "Error in CNCL_GetSendDataPWGRasterCommand\n" );
-		goto onErr2;
+		goto onErr;
 	}
 	GETPRINTCOMMAND = dlsym( libclss, "CNCL_GetPrintCommand" );
 	if ( dlerror() != NULL ) {
 		fprintf( stderr, "Error in CNCL_GetPrintCommand\n" );
-		goto onErr2;
+		goto onErr;
 	}
 	GETSTRINGWITHTAGFROMFILE = dlsym( libclss, "CNCL_GetStringWithTagFromFile" );
 	if ( dlerror() != NULL ) {
 		fprintf( stderr, "Error in CNCL_GetStringWithTagFromFile\n" );
-		goto onErr2;
+		goto onErr;
 	}
 	GETSETPAGECONFIGUARTIONCOMMAND = dlsym( libclss, "CNCL_GetSetPageConfigurationCommand" );
 	if ( dlerror() != NULL ) {
 		fprintf( stderr, "Load Error in CNCL_GetSetPageConfigurationCommand\n" );
-		goto onErr2;
+		goto onErr;
 	}
 	MAKEBJLSETTIMEJOB = dlsym( libclss, "CNCL_MakeBJLSetTimeJob" );
 	if ( dlerror() != NULL ) {
 		fprintf( stderr, "Load Error in CNCL_MakeBJLSetTimeJob\n" );
-		goto onErr2;
+		goto onErr;
 	}
 	GetProtocol = dlsym( libclss, "CNCL_GetProtocol" );
 	if ( dlerror() != NULL ) {
 		fprintf( stderr, "Load Error in CNCL_MakeBJLSetTimeJob\n" );
-		goto onErr2;
+		goto onErr;
 	}
 	ParseCapabilityResponsePrint_HostEnv = dlsym( libclss, "CNCL_ParseCapabilityResponsePrint_HostEnv" );
 	if ( dlerror() != NULL ) {
 		fprintf( stderr, "Load Error in CNCL_ParseCapabilityResponsePrint_HostEnv\n" );
-		goto onErr2;
+		goto onErr;
 	}
 	MakeCommand_StartJob3 = dlsym( libclss, "CNCL_MakeCommand_StartJob3" );
 	if ( dlerror() != NULL ) {
 		fprintf( stderr, "Load Error in CNCL_MakeCommand_StartJob3\n" );
-		goto onErr2;
+		goto onErr;
 	}
 	ParseCapabilityResponsePrint_DateTime = dlsym( libclss, "CNCL_ParseCapabilityResponsePrint_DateTime" );
 	if ( dlerror() != NULL ) {
 		fprintf( stderr, "Load Error in CNCL_ParseCapabilityResponsePrint_DateTime\n" );
-		goto onErr2;
+		goto onErr;
 	}
 	MakeCommand_SetJobConfiguration = dlsym( libclss, "CNCL_MakeCommand_SetJobConfiguration" );
 	if ( dlerror() != NULL ) {
 		fprintf( stderr, "Load Error in CNCL_MakeCommand_SetJobConfiguration\n" );
-		goto onErr2;
+		goto onErr;
 	}
 
 	/* Check Settings */
-	if ( CheckSettings( &Settings ) != 0 ) goto onErr2;
+	if ( CheckSettings( &Settings ) != 0 ) goto onErr;
 
 #if 1
 	/* Dump Settings */
 	DumpSettings( &Settings );
 #endif
 
-	// const char *p_ppd_name = getenv("PPD");
-	CAPABILITY_DATA capability;
 	memset(&capability, '\0', sizeof(CAPABILITY_DATA));
-
 	if( ! GetCapabilityFromPPDFile(p_ppd_name, &capability) ){
-		goto onErr3;
+		goto onErr;
 	}
 
-	int prot = GetProtocol( (char *)capability.deviceID, capability.deviceIDLength );
-
-	if( prot == 2 ){
-		xmlBufSize = GETSTRINGWITHTAGFROMFILE( p_ppd_name, CNCL_FILE_TAG_CAPABILITY, (int *)CNCL_DECODE_EXEC, &xmlBuf );
-
-		unsigned short hostEnv = 0;
-		hostEnv = ParseCapabilityResponsePrint_HostEnv( xmlBuf, xmlBufSize );
-
-		/* Set JobID */
-		strncpy( jobID, CN_START_JOBID2, sizeof(CN_START_JOBID2) );
-
-		/* Allocate Buffer */
-		bufSize = sizeof(char) * CN_BUFSIZE;
-
-		if ( (bufTop = malloc( bufSize )) == NULL ) goto onErr2;
-
-		/* Write StartJob Command */
-		int ret = 0;
-		ret = MakeCommand_StartJob3( hostEnv, uuid, jobID, bufTop, bufSize, &writtenSize );
-
-		if ( ret != 0 ) {
-			fprintf( stderr, "Error in CNCL_GetPrintCommand\n" );
-			goto onErr2;
-		}
-
-		/* WriteData */
-		if ( (retSize = write( 1, bufTop, writtenSize )) != writtenSize ) goto onErr2;
-
-
-		char dateTime[15];
-		memset(dateTime, '\0', sizeof(dateTime));
-
-		ret = ParseCapabilityResponsePrint_DateTime( xmlBuf, xmlBufSize );
-
-		if( ret == 2 ){
-			time_t timer = time(NULL);
-			struct tm *date = localtime(&timer);
-
-			sprintf(dateTime, "%d%02d%02d%02d%02d%02d",
-				date->tm_year+1900, date->tm_mon+1, date->tm_mday,
-				date->tm_hour, date->tm_min, date->tm_sec);
-
-			if ( (tmpBuf = malloc( bufSize )) == NULL ) goto onErr2;
-
-			ret = MakeCommand_SetJobConfiguration( jobID, dateTime, tmpBuf, bufSize, &writtenSize );
-
-			/* WriteData */
-			if ( (retSize = write( 1, tmpBuf, writtenSize )) != writtenSize ) goto onErr2;
-
-			// writtenSize += tmpWrittenSize;
-		}
-	}
-	else{
-		/* Set JobID */
-		strncpy( jobID, CN_START_JOBID, sizeof(CN_START_JOBID) );
-
-		/* OutputSetTime */
-		if ( OutputSetTime( 1, jobID ) != 0 ) goto onErr2;
-
-
-		/* Allocate Buffer */
-		bufSize = sizeof(char) * CN_BUFSIZE;
-		if ( (bufTop = malloc( bufSize )) == NULL ) goto onErr2;
-
-		/* Write StartJob Command */
-		if ( GETPRINTCOMMAND( bufTop, bufSize, &writtenSize_long, jobID, CNCL_COMMAND_START1 ) != 0 ) {
-			fprintf( stderr, "Error in CNCL_GetPrintCommand\n" );
-			goto onErr2;
-
-		}
-
-		/* WriteData */
-		if ( (retSize = write( 1, bufTop, writtenSize_long )) != writtenSize_long ) goto onErr2;
+	/* Get jobID */
+	if(GetJobId(jobID, capability) != 0){
+		goto onErr;
 	}
 
-	/* Write SetConfiguration Command */
-	if ( (xmlBufSize = GETSTRINGWITHTAGFROMFILE( p_ppd_name, CNCL_FILE_TAG_CAPABILITY, (int*)CNCL_DECODE_EXEC, &xmlBuf )) < 0 ){
-		DEBUG_PRINT2( "[tocanonij] p_ppd_name : %s\n", p_ppd_name );
-		DEBUG_PRINT2( "[tocanonij] xmlBufSize : %d\n", xmlBufSize );
-		fprintf( stderr, "Error in CNCL_GetStringWithTagFromFile\n" );
-		goto onErr3;
-	}
-
-	if ( GETSETCONFIGURATIONCOMMAND( &Settings, jobID, bufSize, (void *)xmlBuf, xmlBufSize, bufTop, &writtenSize_long ) != 0 ){
-		fprintf( stderr, "Error in CNCL_GetSetConfigurationCommand\n" );
-		goto onErr3;
-	}
-	/* WriteData */
-	retSize = write( 1, bufTop, writtenSize_long );
-
-	/* Write Page Data */
-	while ( 1 ) {
-		int readBytes = 0;
-		int writeBytes;
-		char *pCurrent;
-		CNDATA CNData;
-		long readSize = 0;
-		unsigned short next_page;
-
-		memset( &CNData, 0, sizeof(CNDATA) );
-
-		/* read magic number */
-		readBytes = read( fd, &CNData, sizeof(CNDATA) );
-		if ( readBytes > 0 ){
-			if ( CNData.magic_num != MAGIC_NUMBER_FOR_CNIJPWG ){
-				fprintf( stderr, "Error illeagal MagicNumber\n" );
-				goto onErr3;
-			}
-			if ( CNData.image_size < 0 ){
-				fprintf( stderr, "Error illeagal dataSize\n" );
-				goto onErr3;
-			}
-		}
-		else if ( readBytes < 0 ){
-			if ( errno == EINTR ) continue;
-			fprintf( stderr, "DEBUG:[tocanonij] tocnij read error, %d\n", errno );
-			goto onErr3;
-		}
-		else {
-			DEBUG_PRINT( "DEBUG:[tocanonij] !!!DATA END!!!\n" );
-			break; /* data end */
+	if((Settings.colormode == CNCL_PSET_COLORMODE_COLOR) && 
+	   (optimization == 1 )){
+		/* Create cache file */
+		CreateCacheFile(cnijtmp_fds);
+		if(cnijtmp_fds[0] == -1 || cnijtmp_fds[1] == -1){
+			goto onErr;
 		}
 
-		/* Write Next Page Info */
-		if ( CNData.next_page ) {
-			next_page = CNCL_PSET_NEXTPAGE_ON;
+		/* Write data to cache file */
+		if(WriteCacheFile(fd, cnijtmp_fds, jobID, &jobColorMode) != 0){
+			goto onErr;
 		}
-		else {
-			next_page = CNCL_PSET_NEXTPAGE_OFF;
+
+		if(jobColorMode == COLOR_MODE_COLOR){
+			cnijtmp_fd = cnijtmp_fds[0];
+		} else if (jobColorMode == COLOR_MODE_GRAY){
+			Settings.colormode = CNCL_PSET_COLORMODE_MONO;
+			cnijtmp_fd = cnijtmp_fds[1];
+		} else{
+			goto onErr;
 		}
-		if ( GETSETPAGECONFIGUARTIONCOMMAND( jobID, next_page, bufTop, bufSize, &writtenSize_long ) != 0 ) {
-			fprintf( stderr, "Error in CNCL_GetPrintCommand\n" );
-			goto onErr3;
+		DEBUG_PRINT2( "[tocanonij] jobColorMode:%d\n", jobColorMode);
+
+		/* Write header data to out port*/
+		if(WriteHeader(1, jobID, uuid, &Settings, capability) != 0){
+			goto onErr;
 		}
-		/* WriteData */
-		if ( (retSize = write( 1, bufTop, writtenSize_long )) != writtenSize_long ) goto onErr3;
 
-		/* Write SendData Command */
-		memset(	bufTop, 0x00, bufSize );
-		readSize = CNData.image_size;
-		if ( GETSENDDATAPWGRASTERCOMMAND( jobID, readSize, bufSize, bufTop, &writtenSize_long ) != 0 ) {
-			DEBUG_PRINT( "Error in CNCL_GetSendDataJPEGCommand\n" );
-			goto onErr3;
+		/* Get data from cache file and write it to out port */
+		if(ReplayCacheFile(cnijtmp_fd, 1) != 0){
+			goto onErr;
 		}
-		/* WriteData */
-		retSize = write( 1, bufTop, writtenSize_long );
 
-		while( readSize ){
-			pCurrent = bufTop;
-
-			if ( readSize - bufSize > 0 ){
-				readBytes = read( fd, bufTop, bufSize );
-				DEBUG_PRINT2( "[tocanonij] PASS tocanonij READ1<%d>\n", readBytes );
-				if ( readBytes < 0 ) {
-					if ( errno == EINTR ) continue;
-				}
-				readSize -= readBytes;
-			}
-			else {
-				readBytes = read( fd, bufTop, readSize );
-				DEBUG_PRINT2( "[tocanonij] PASS tocanonij READ2<%d>\n", readBytes );
-				if ( readBytes < 0 ) {
-					if ( errno == EINTR ) continue;
-				}
-				readSize -= readBytes;
-			}
-
-			do {
-				writeBytes = write( 1, pCurrent, readBytes );
-				DEBUG_PRINT2( "[tocanonij] PASS tocanonij WRITE<%d>\n", writeBytes );
-				if( writeBytes < 0){
-					if ( errno == EINTR ) continue;
-					goto onErr3;
-				}
-				readBytes -= writeBytes;
-				pCurrent += writeBytes;
-			} while( writeBytes > 0 );
+		/*Write tail data to out port */
+		if(WriteTail(1, jobID) != 0){
+			goto onErr;
+		}
+		
+	} else {
+		/* Write header data */
+		if(WriteHeader(1, jobID, uuid, &Settings, capability) != 0){
+			goto onErr;
+		}
 
+		/* Write page Data */
+		if(WritePages(fd, 1, jobID) != 0){
+			goto onErr;
+		}
+		
+		/* Write tail data */
+		if(WriteTail(1, jobID) != 0){
+			goto onErr;
 		}
 	}
 
-	/* CNCL_GetPrintCommand */
-	if ( GETPRINTCOMMAND( bufTop, bufSize, &writtenSize_long, jobID, CNCL_COMMAND_END ) != 0 ) {
-		DEBUG_PRINT( "Error in CNCL_GetPrintCommand\n" );
-		goto onErr3;
-
-	}
-	/* WriteData */
-	retSize = write( 1, bufTop, writtenSize_long );
-	DEBUG_PRINT( "[tocanonij] to_cnijf <end>\n" );
-
 	result = 0;
-onErr3:
-	if ( xmlBuf != NULL ){
-		free( xmlBuf );
-	}
 
-onErr2:
-	if ( bufTop != NULL ){
-		free( bufTop );
-	}
-
-onErr1:
+onErr:
 	if ( libclss != NULL ) {
 		dlclose( libclss );
 	}
 
-	if( tmpBuf != NULL ){
-		free(tmpBuf);
+	if(cnijtmp_fds[0] != -1){
+		close(cnijtmp_fds[0]);
 	}
+
+	if(cnijtmp_fds[1] != -1){
+		close(cnijtmp_fds[1]);
+	}
+
 	return result;
 }
 
