@@ -19,10 +19,18 @@
 #define MAGIC_NUMBER_FOR_CNIJPWG	0x12340001
 #define MAGIC_NUMBER_FOR_CNIJJPEG	0x12340002
 
+enum ColorMode{
+	COLOR_MODE_UNKNOWN = 0,
+	COLOR_MODE_COLOR,
+	COLOR_MODE_GRAY
+};
+
 typedef struct cndata {
 	long	magic_num;
 	long	image_size;
 	short	next_page;
 	short	page_num;
+	enum ColorMode	jobColorMode;
+	enum ColorMode	pageColorMode;
 	long	reserve[13];
 } CNDATA, *LPCNDATA;
